﻿// --------------------------------------------------------------
// Beispiel: ...\Kapitel 23\Texteditor
// --------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using System.IO;

namespace Texteditor
{
  public partial class Form1 : Form
  {
    // Startseite zum Drucken
		private int startSeite;
		// Anzahl der Druckseiten
		private int anzahlSeiten;
		// aktuelle Seitenzahl
		private int seitenNummer;
		// zu druckender Text
		private string strPrintText;
    // der Name der geöffneten Datei
    private string strDateiName;

    public Form1()
    {
      InitializeComponent();
      printDialog1.PrinterSettings = new PrinterSettings();
    }

    private void mnuDrucken_Click(object sender, EventArgs e)
    {
      printDialog1.AllowSomePages = true;
      if (printDialog1.ShowDialog() == DialogResult.OK)
      {
        printDocument1.DocumentName = "Dokument";
        // Startwerte abhängig vom zu druckenden Text initialisieren
        switch (printDialog1.PrinterSettings.PrintRange)
        {
          case PrintRange.AllPages:
            strPrintText = textBox1.Text;
            startSeite = 1;
            anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
            break;
          case PrintRange.SomePages:
            strPrintText = textBox1.Text;
            startSeite = printDialog1.PrinterSettings.FromPage;
            anzahlSeiten = printDialog1.PrinterSettings.ToPage - startSeite + 1;
            break;
        }
        // Drucken starten
        seitenNummer = 1;
        printDocument1.Print();
      }
    }

    private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
    {
      StringFormat stringFormat = new StringFormat();
      RectangleF rectFPapier, rectFText;
      int intChars, intLines;

      // Ermitteln des Rectangles, das den gesamten Druckbereich
      // beschreibt (inklusive Kopf- und Fusszeile)
      rectFPapier = e.MarginBounds;

      // Ermitteln des Rectangles, das den Bereich für den
      // Text beschreibt (ausschließlich Kopf- und Fusszeile)
      rectFText = RectangleF.Inflate(rectFPapier, 0, -2 * textBox1.Font.GetHeight(e.Graphics));

      // eine gerade Anzahl von Druckzeilen ermitteln
      int anzahlZeilen = (int)Math.Floor(rectFText.Height / textBox1.Font.GetHeight(e.Graphics));

      // die Höhe die textbeinhaltenden Rechtecks festlegen, damit die
      // letzte Druckzeile nicht abgeschnitten wird
      rectFText.Height = anzahlZeilen * textBox1.Font.GetHeight(e.Graphics);

      // das StringFormat-Objekt festlegen, um den Text in einem Rechteck
      // anzuzeigen - Text bis zum nächstliegenden Wort verkürzen
      stringFormat.Trimming = StringTrimming.Word;

      //	legt die Druckstartseite fest, wenn es sich nicht um die
      // erste Dokumentenseite handelt
      while ((seitenNummer < startSeite) && (strPrintText.Length > 0))
      {
        e.Graphics.MeasureString(strPrintText, textBox1.Font, rectFText.Size, stringFormat, out intChars, out intLines);
        strPrintText = strPrintText.Substring(intChars);
        seitenNummer++;
      }

      // Druckjob beenden, wenn es keinen Text zum Drucken mehr gibt
      if (strPrintText.Length == 0)
      {
        e.Cancel = true;
        return;
      }

      // den Text an das Graphics-Objekt übergeben
      e.Graphics.DrawString(strPrintText, textBox1.Font, Brushes.Black, rectFText, stringFormat);

      // Text für die nächste Seite
      // intChars - Anzahl der Zeichen in der Zeichenfolge
      // intLines - Anzahl der Zeilen in der Zeichenfolge
      e.Graphics.MeasureString(strPrintText, textBox1.Font, rectFText.Size, stringFormat, out intChars, out intLines);
      strPrintText = strPrintText.Substring(intChars);

      // StringFormat restaurieren
      stringFormat = new StringFormat();

      // Dateiname in der Kopfzeile anzeigen
      stringFormat.Alignment = StringAlignment.Center;
      e.Graphics.DrawString(this.strDateiName, textBox1.Font, Brushes.Black, rectFPapier, stringFormat);

      // Seitennummer in der Fusszeile anzeigen
      stringFormat.LineAlignment = StringAlignment.Far;
      e.Graphics.DrawString("Page " + seitenNummer, textBox1.Font, Brushes.Black, rectFPapier, stringFormat);

      // ermitteln, ob weitere Seiten zu drucken sind
      seitenNummer++;
      e.HasMorePages = (strPrintText.Length > 0) && (seitenNummer < startSeite + anzahlSeiten);

      // Neuinitialisierung
      if (!e.HasMorePages)
      {
        strPrintText = textBox1.Text;
        startSeite = 1;
        anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
        seitenNummer = 1;
      }
    }

    private void mnuOpen_Click(object sender, EventArgs e)
    {
      openFileDialog1.Filter = "Text Documents(*.txt)|*.txt|All Files(*.*)|*.*"; ;
      openFileDialog1.FileName = "*.txt";
      StreamReader sr;
      if (openFileDialog1.ShowDialog() == DialogResult.OK)
        try
        {
          this.strDateiName = openFileDialog1.FileName;
          sr = new StreamReader(this.strDateiName);
          textBox1.Text = sr.ReadToEnd();
          sr.Close();
        }
        catch (Exception exc)
        {
          MessageBox.Show(exc.Message, "Texteditor", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
          return;
        }

      this.Text += this.strDateiName;
      textBox1.SelectionStart = 0;
      textBox1.SelectionLength = 0;
    }

    private void mnuPrintPreview_Click(object sender, EventArgs e)
    {
      printDialog1.AllowSomePages = true;
      strPrintText = textBox1.Text;
      startSeite = 1;
      anzahlSeiten = printDialog1.PrinterSettings.MaximumPage;
      seitenNummer = 1;
      printPreviewDialog1.ShowDialog();
    }

    private void mnuPageSetup_Click(object sender, EventArgs e)
    {
      pageSetupDialog1.ShowDialog();
    }
  }
}